//Przemysaw Linke H1ISI

/**set_symmetric_difference - funkcja naley do zbioru podstawowych operacji
 *                           wykonywanych na posortowanym cigu.
 */

/**Funkcja set_symmetric_difference tworzy posortowany cig bdcy rnic
 *symetryczn dwch innych cigw. Oznacza to, e w powstaym cigu znajduj si
 *kopie elementw nalecych do pierwszego z zakresw [first1, last1) ale nie
 *nalece do drugiego [first2, last2), oraz na odwrt, elementy nalece do  
 *drugiego z zakresw [first2, last2), ale nie nalece do pierwszego [first1, last1).
 *Funkcja zwiera iterator wskazujcy na ostatni element utworzonego cigu.
 */

/**Nazwa funkcji set_symmetric_difference jest przeciona. Funkcja wystpuje
 *w dwch wersjach. Rni si one sposobem rozrniania czy jeden element jest
 *mniejszy ni drugi. Pierwsza z wersji do porwnania uywa operatora <,
 *druga funkcji comp() dostarczonej przez uytkownika.
 *
 *template <class InputIterator1, class InputIterator2, class OutputIterator>
 *OutputIterator set_symmetric_difference(InputIterator1 first1,
 *                                        InputIterator1 last1,
 *                                        InputIterator2 first2,
 *                                        InputIterator2 last2,
 *                                        OutputIterator result);
 *
 *template <class InputIterator1, class InputIterator2, class OutputIterator,
 *          class StrictWeakOrdering>
 *OutputIterator set_symmetric_difference(InputIterator1 first1,
 *                                        InputIterator1 last1,
 *                                        InputIterator2 first2,
 *                                        InputIterator2 last2,
 *                                        OutputIterator result,
 *                                        StrictWeakOrdering comp);
 *
 *
 *Argumenty
 *
 *-Dla obu wersji
 *    -InputIterator1 jest typu Input Iterator - dla pierwszego cigu
 *    -InputIterator2 jest typu Input Iterator - dla drugiego cigu
 *    -OutputIterator jest typu Output Iterator - dla cigu wynikowego
 *    -InputIterator1 i InputIterator2 maj ten sam typ wartoci
 *    -warto InputIterator-w musi by konwertowana na typ wartoci dla
 *      OutputIterator-a.
 *
 *-Dla pierwszej wersji
 *    -Typ wartoci InputIterator-a musi da si porwnywa przy pomocy operatora <
 *    -Kolejno obiektw w InputIterator1 jest uporzdkowana zgodnie z relacj
 *      czciowego porzdku
 *
 *-Dla drugiej wersji
 *    -StrictWeakOrdering realizuje relacj porzdku czciowego
 *    -Typ wartoci InputIterator1-a jest konwertowalny na typ argumentw funkcji
 *      StrictWeakOrdering
 */

//PRZYKLAD

#include <iostream>
#include <istream>
#include <iterator>
#include <algorithm>
#include <set>

using namespace std;

 int main()
 {
   //Tworzymy dwa cigi
   
   //Pierwszy skada si z elementw 1 2 4 5 8 9
   int zbior1[] = {1,2,4,5,8,9};
   set<int, less<int> > ciag1 (zbior1, zbior1+6);
   
   //Drugi skada si z elementw 1 3 4 7 9      
   int zbior2[] = {1,3,4,7,9};
   set<int, less<int> > ciag2(zbior2, zbior2+5);
   
   //Kontener na wynik
   set<int, less<int> > wynik;                    

   //Utworzenie insert_iterator dla wyniku
   insert_iterator<set<int, less<int> > > res_ins(wynik, wynik.begin());

   //Demonstracja dziaania set_symmetric_difference
   cout << "Roznica symetryczna zbioru:" << endl << "{";
   
   //Wypisanie pierwszego zbioru
   //{1,2,4,5,8,9}
   copy(ciag1.begin(),ciag1.end(), ostream_iterator<int,char>(cout," "));
   
   cout << "} ze zbiorem {";
   
   //Wypisanie drugiego zbioru
   //{1,3,4,7,9}
   copy(ciag2.begin(),ciag2.end(), ostream_iterator<int,char>(cout," "));
   
   cout << "} jest rowna " << endl << "{";
   
   //Wykorzystanie funkcji set_symmetric_difference
   set_symmetric_difference(ciag2.begin(), ciag2.end(), ciag1.begin(), ciag1.end(), res_ins);

   //Wypisanie wyniku
   //{2 3 5 7 8 } Jak atwo zauway w wynikowym cigu brak 1, 4 i 9.                        
   copy(wynik.begin(),wynik.end(), ostream_iterator<int,char>(cout," "));
        
   cout << "}" << endl << endl;

   return 0;
 }

/**Gdy elementy nie powtarzaj si, algorytm tworzy sum dwch zbiorw A\B i B\A.
 *Przeciwny przypadek jest bardziej skomplikowany, poniewa wejciowe cigi mog  
 *zawiera powielone elementy. Jeli dany element wystpuje m razy w pierwszym 
 *cigu [first1, last1) i n razy w drugim [first2, last2) to w cigu wynikowym
 *wystpi |m-n| razy.
 */

/**Algorytm ma liniow zoono. Zoono jest ona okrelana wzgldem iloci porwna.
 *Jeli ktrykolwiek z cigw jest pusty nie zachodzi adne porwnanie, w przeciwnym
 *wypadku zachodzi co najwyej 2*((last1-first1) +  (last2-first2)) porwna.
 */

/**Wynik dziaania funkcji jest nieokrelony jeli zakres wynikowego cigu
 *przekroczy zakres ktregokolwiek z oryginalnych cigw.
 */
 
/**Algorytm jest stabilny tzn. wzgldne pooenie elementw w kadym z wejciowych
 *cigw jest zachowane.
 */


